!=========================================================================
!                   MODULE FOR THE EVAP/COND
!
! NOTE: The rhodust(:,:,:) array is actually a number density! In the 
!       paper it is N_k,l. 
!=========================================================================
module userdef_module
use namelist_module
use variables_module

integer :: userdef_nscond,userdef_nsrefr,userdef_nstepmax
double precision, allocatable :: userdef_c0(:,:),userdef_cm(:,:)
double precision, allocatable :: userdef_e0(:,:),userdef_ep(:,:)
double precision, allocatable :: userdef_nd(:)

contains

!-------------------------------------------------------------------------
!                        SET DEFAULTS 
!-------------------------------------------------------------------------
subroutine userdef_set_defaults()
  implicit none
  userdef_nscond = 1
  userdef_nstepmax = 10000
  return
end subroutine userdef_set_defaults

!-------------------------------------------------------------------------
!             PARAMETER READING ROUTINE FOR USERDEFINED MODULE
!-------------------------------------------------------------------------
subroutine userdef_parse_input()
  implicit none
  call parse_input_integer('userdef_nscond@               ',userdef_nscond)
  call parse_input_integer('userdef_nstepmax@             ',userdef_nstepmax)
  return
end subroutine userdef_parse_input

!-------------------------------------------------------------------------
!                 READING ROUTINE FOR USERDEFINED MODULE
!-------------------------------------------------------------------------
subroutine userdef_readall()
  implicit none
  return
end subroutine userdef_readall

!-------------------------------------------------------------------------
!                 CHECKING ROUTINE FOR USERDEFINED MODULE
!-------------------------------------------------------------------------
subroutine userdef_checkall()
  implicit none
  integer :: i,is,im,iss
  double precision :: a,b
  !
  if(userdef_nscond.lt.1) stop 999
  !
  ! Split the total amount of dust mass bins into nrspec x nscond 
  !
  i = dust_nrspec / userdef_nscond
  a = i
  b = dust_nrspec / ( 1.d0 * userdef_nscond )
  if(a.ne.b) then
     write(*,*) 'ERROR: The total nr of dust species is not divisible by '
     write(*,*) '       userdef_nscond...'
     stop
  endif
  userdef_nsrefr = dust_nrspec / userdef_nscond
  !
  ! Check that the masses all increase with mu
  ! 
  do is=1,userdef_nsrefr
     do im=2,userdef_nscond
        iss = (is-1)*userdef_nscond + im
        if(dust_m(iss).le.dust_m(iss-1)) then
           write(*,*) 'ERROR: Dust mass does not increase everywhere with mu.'
           stop
        endif
     enddo
  enddo
  !
  ! Allocate some intermediate arrays
  !
  allocate(userdef_c0(1:userdef_nscond,1:userdef_nsrefr))
  allocate(userdef_cm(1:userdef_nscond,1:userdef_nsrefr))
  allocate(userdef_e0(1:userdef_nscond,1:userdef_nsrefr))
  allocate(userdef_ep(1:userdef_nscond,1:userdef_nsrefr))
  allocate(userdef_nd(1:dust_nrspec))
  return
end subroutine userdef_checkall

!-------------------------------------------------------------------------
!                 WRITING ROUTINE FOR USERDEFINED MODULE
!-------------------------------------------------------------------------
subroutine userdef_writeframe(itime)
  implicit none
  integer :: itime
  return
end subroutine userdef_writeframe

!-------------------------------------------------------------------------
!                   OPERATOR-SPLITTED `REACTION RATES'
!-------------------------------------------------------------------------
subroutine userdef_do_reactions()
  implicit none
  !
  double precision :: peq,alp,surfbare,surfcm,surftop,coef,mh2o,tg
  double precision :: src_vapor,cnd_vapor,dt,tcond_vap,tcond_dust,nvap
  double precision :: tcondevap_dust,dummy,c0,e,tcondevap,cfl,time,timenew
  integer :: ir,iz,is,im,iss,istep
  doubleprecision :: pi,mp,kk
  parameter(pi  = 3.1415926535897932385d0)
  parameter(mp  = 1.6726d-24)    ! Mass of proton          [g]
  parameter(kk  = 1.3807d-16)    ! Bolzmann's constant     [erg/K]
  parameter(cfl = 0.5)
  !
  ! Set the sticking coefficient, which we take globally constant for now
  !
  alp = 0.9d0
  !
  ! Set the mass of the vapor molecule
  !
  mh2o = (16.d0+2.d0)*mp
  !
  ! Loop over the radial and vertical coordinate
  !
  do ir=1,grid_nr
     do iz=1,grid_nz
        !
        ! Get the gas temperature
        !
        tg = tgas(iz,ir)
        !
        ! Calculate the equilibrium vapor pressure
        !
        peq = exp(-(6070.d0/tg) + 30.86d0)
        !
        ! Compute the coefficient alp / sqrt(2*pi*m*k*T)
        !
        coef = alp / sqrt(2*pi*mh2o*kk*tg)
        !
        ! Compute the source and sink coefficients for the N_ik, i.e.
        ! the left and right `velocities' in mass-space. 
        !
        c0 = coef*mh2o*kk*tg
        e  = coef*mh2o*peq
        if(c0.lt.0.d0) stop 901
        if(e.lt.0.d0) stop 902
        do is=1,userdef_nsrefr
           !
           ! First for the main mu-grid 
           !
           do im=2,userdef_nscond-1
              iss               = (is-1)*userdef_nscond + im
              userdef_c0(im,is) = -4*pi*dust_a(iss)**2*c0/(dust_m(iss+1)-dust_m(iss))
              userdef_cm(im,is) =  4*pi*dust_a(iss-1)**2*c0/(dust_m(iss)-dust_m(iss-1))
              userdef_e0(im,is) = -4*pi*dust_a(iss)**2*e/(dust_m(iss)-dust_m(iss-1))
              userdef_ep(im,is) =  4*pi*dust_a(iss+1)**2*e/(dust_m(iss+1)-dust_m(iss))
           enddo
           !
           ! Then boundary condition at im=1: bare grains cannot evaporate
           !
           im                = 1
           iss               = (is-1)*userdef_nscond + im
           userdef_c0(im,is) = -4*pi*dust_a(iss)**2*c0/(dust_m(iss+1)-dust_m(iss))
           userdef_cm(im,is) = 0.d0
           userdef_e0(im,is) = 0.d0
           userdef_ep(im,is) =  4*pi*dust_a(iss+1)**2*e/(dust_m(iss+1)-dust_m(iss))
           !
           ! Then boundary condition at im=userdef_nscond: maximum size
           ! core-mantel grains cannot acquire more vapor (this is an 
           ! artificial limit originating from the numerical procedure).
           !
           im                = userdef_nscond
           iss               = (is-1)*userdef_nscond + im
           userdef_c0(im,is) = 0.d0
           userdef_cm(im,is) =  4*pi*dust_a(iss-1)**2*c0/(dust_m(iss)-dust_m(iss-1))
           userdef_e0(im,is) = -4*pi*dust_a(iss)**2*e/(dust_m(iss)-dust_m(iss-1))
           userdef_ep(im,is) = 0.d0
        enddo
        !
        ! Do the sub-time-stepping for the evaporation/condensation
        !
        time = 0.d0
        istep = 1
        do while(time.lt.time_dt)
           !
           ! Check if we are not looping forever
           !
           if(istep.gt.userdef_nstepmax) then
              write(*,*) 'ERROR: Time substepping overflow...'
              stop 701
           endif
           !
           ! Get the vapor number density
           !
           nvap = rhochem(iz,1,ir)
           !
           ! Get the dust number densities
           !
           userdef_nd(:) = rhodust(iz,:,ir)
           !
           ! Compute the total surface area of bare and core-mantel grains
           !
           surfbare = 0.d0
           surfcm   = 0.d0
           surftop  = 0.d0
           do is=1,userdef_nsrefr
              iss  = (is-1)*userdef_nscond + 1
              surfbare = surfbare + userdef_nd(iss) * 4 * pi * dust_a(iss)**2
              do im=2,userdef_nscond-1
                 iss = iss + 1
                 surfcm = surfcm + userdef_nd(iss) * 4 * pi * dust_a(iss)**2
              enddo
              iss = iss + 1
              surftop = surftop + userdef_nd(iss) * 4 * pi * dust_a(iss)**2
           enddo
           !
           ! Compute the source term and condensation coefficient 
           ! for the gas-phase vapor. Note that bare grains cannot
           ! evaporate further and grains with the maximum ice layer
           ! (a numerical limit because of the largest mass bin of
           ! the core-mantel grains, which should never be reached,
           ! or else the results are wrong) should not grow further
           ! through further condensation. Hence surfbare resp. surftop.
           !
           src_vapor = coef * ( surfcm + surftop ) * peq 
           cnd_vapor = coef * ( surfcm + surfbare ) * kk * tg
           !
           ! Compute the time scale for vapor density change based on
           ! condensation only. This is the time scale of the vapor.
           !
           tcond_vap = 1.d0 / cnd_vapor
           !
           ! Find the time scale for the dust growth/shrinking
           !
           tcondevap_dust = 1d99
           do is=1,userdef_nsrefr
              do im=1,userdef_nscond
                 dummy = 1.d0 / ( 1d-99 -                          &
                      userdef_c0(im,is)*nvap - userdef_e0(im,is) ) 
                 if(dummy.lt.tcondevap_dust) then
                    tcondevap_dust = dummy
                 endif
              enddo
           enddo
           !
           ! Compute the time step for the sub-stepping
           !
           tcondevap = min(tcondevap_dust,tcond_vap)
           dt        = cfl*tcondevap
           if(dt.le.0.d0) then
              write(*,*) 'ERROR: Dt is <=0:   dt=',dt
              write(*,*) tcondevap_dust,tcond_vap
              stop 333
           endif
           !
           ! Make sure that the time step is right
           !
           timenew = time + dt
           if(timenew.gt.time_dt) then
              dt      = time_dt - time
              timenew = time_dt
           endif
           !
           ! Now do a simplistic integration of the equation for 
           ! the vapor number density
           !
!#########################
!           rhochem(iz,1,ir) = nvap + dt*(src_vapor-cnd_vapor*nvap)
!#########################
           !
           ! Do a simplistic integration of the equation for 
           ! the dust number densities
           !
           do is=1,userdef_nsrefr
              do im=2,userdef_nscond-1
                 iss                  = (is-1)*userdef_nscond + im
                 rhodust(iz,iss,ir)   = userdef_nd(iss) + dt * (  &
                     nvap*userdef_c0(im,is)*userdef_nd(iss) +     &
                     nvap*userdef_cm(im,is)*userdef_nd(iss-1) +   &
                     userdef_e0(im,is)*userdef_nd(iss) +          &
                     userdef_ep(im,is)*userdef_nd(iss+1) )
              enddo
              im    = 1
              iss   = (is-1)*userdef_nscond + im
              rhodust(iz,iss,ir)   = userdef_nd(iss) + dt * (     &
                   nvap*userdef_c0(im,is)*userdef_nd(iss) +       &
                   userdef_ep(im,is)*userdef_nd(iss+1) )
              im    = userdef_nscond
              iss   = (is-1)*userdef_nscond + im
              rhodust(iz,iss,ir)   = userdef_nd(iss) + dt * (     &
                   nvap*userdef_cm(im,is)*userdef_nd(iss-1) +     &
                   userdef_e0(im,is)*userdef_nd(iss) )
           enddo
           !
           ! Do basic checks, and replaced numbers that become smaller
           ! than 1d-98 with 0.d0
           !
           if(rhochem(iz,1,ir).lt.0.d0) stop 661
           do iss=1,dust_nrspec
              if(rhodust(iz,iss,ir).lt.0.d0) stop 662
              if(rhodust(iz,iss,ir).lt.1.d-98) rhodust(iz,iss,ir)=0.d0
           enddo
           !
           ! Update the local time
           !
           time  = timenew
           istep = istep + 1
        enddo
        !
     enddo
  enddo
  !
end subroutine userdef_do_reactions

!-------------------------------------------------------------------------
!                          CLEANUP
!-------------------------------------------------------------------------
subroutine userdef_cleanup()
  implicit none
  if(allocated(userdef_c0)) deallocate(userdef_c0)
  if(allocated(userdef_cm)) deallocate(userdef_cm)
  if(allocated(userdef_e0)) deallocate(userdef_e0)
  if(allocated(userdef_ep)) deallocate(userdef_ep)
  if(allocated(userdef_nd)) deallocate(userdef_nd)
  !
end subroutine userdef_cleanup



end module userdef_module
