!=========================================================================
!                   MODULE FOR THE EVAP/COND
!
! NOTE: The rhodust(:,:,:) array is actually a number density! In the 
!       paper it is N_k,l. 
!=========================================================================
module userdef_module
use namelist_module
use variables_module

integer :: userdef_nscond,userdef_nsrefr,userdef_nstepmax
double precision, allocatable :: userdef_velo(:,:)
double precision, allocatable :: userdef_flux(:,:)
double precision, allocatable :: userdef_flsrc(:,:)
double precision, allocatable :: userdef_mi(:,:)
double precision, allocatable :: userdef_ai(:,:)
double precision, allocatable :: userdef_ndust(:,:)
double precision :: userdef_cfl,userdef_dneq

contains

!-------------------------------------------------------------------------
!                        SET DEFAULTS 
!-------------------------------------------------------------------------
subroutine userdef_set_defaults()
  implicit none
  userdef_nscond = 1
  userdef_nstepmax = 10000
  userdef_cfl = 0.5
  userdef_dneq = 1d-8
  return
end subroutine userdef_set_defaults

!-------------------------------------------------------------------------
!             PARAMETER READING ROUTINE FOR USERDEFINED MODULE
!-------------------------------------------------------------------------
subroutine userdef_parse_input()
  implicit none
  call parse_input_integer('userdef_nscond@               ',userdef_nscond)
  call parse_input_integer('userdef_nstepmax@             ',userdef_nstepmax)
  call parse_input_double ('userdef_cfl@                  ',userdef_cfl)
  call parse_input_double ('userdef_dneq@                 ',userdef_dneq)
  return
end subroutine userdef_parse_input

!-------------------------------------------------------------------------
!                 READING ROUTINE FOR USERDEFINED MODULE
!-------------------------------------------------------------------------
subroutine userdef_readall()
  implicit none
  return
end subroutine userdef_readall

!-------------------------------------------------------------------------
!                 CHECKING ROUTINE FOR USERDEFINED MODULE
!-------------------------------------------------------------------------
subroutine userdef_checkall()
  implicit none
  integer :: i,is,im,iss
  double precision :: a,b
  !
  if(userdef_nscond.lt.1) stop 999
  !
  ! Split the total amount of dust mass bins into nrspec x nscond 
  !
  i = dust_nrspec / userdef_nscond
  a = i
  b = dust_nrspec / ( 1.d0 * userdef_nscond )
  if(a.ne.b) then
     write(*,*) 'ERROR: The total nr of dust species is not divisible by '
     write(*,*) '       userdef_nscond...'
     stop
  endif
  userdef_nsrefr = dust_nrspec / userdef_nscond
  !
  ! Check that the masses all increase with mu
  ! 
  do is=1,userdef_nsrefr
     do im=2,userdef_nscond
        iss = (is-1)*userdef_nscond + im
        if(dust_m(iss).le.dust_m(iss-1)) then
           write(*,*) 'ERROR: Dust mass does not increase everywhere with mu.'
           stop
        endif
     enddo
  enddo
  !
  ! Allocate some intermediate arrays
  !
  allocate(userdef_velo(1:userdef_nscond+1,1:userdef_nsrefr))
  allocate(userdef_flux(1:userdef_nscond+1,1:userdef_nsrefr))
  allocate(userdef_flsrc(1:userdef_nscond,1:userdef_nsrefr))
  allocate(userdef_mi(1:userdef_nscond+1,1:userdef_nsrefr))
  allocate(userdef_ai(1:userdef_nscond+1,1:userdef_nsrefr))
  allocate(userdef_ndust(1:userdef_nscond,1:userdef_nsrefr))
  !
  ! Compute the userdef_mi and userdef_ai
  !
  do is=1,userdef_nsrefr
     do im=2,userdef_nscond
        iss = (is-1)*userdef_nscond + im
        userdef_mi(im,is) = sqrt( dust_m(iss) * dust_m(iss-1) )
        userdef_ai(im,is) = sqrt( dust_a(iss) * dust_a(iss-1) )
     enddo
     userdef_mi(1,is) = userdef_mi(2,is)**2 / userdef_mi(3,is)
     userdef_mi(userdef_nscond+1,is) = userdef_mi(userdef_nscond,is)**2 / &
          userdef_mi(userdef_nscond-1,is)
     userdef_ai(1,is) = userdef_ai(2,is)**2 / userdef_ai(3,is)
     userdef_ai(userdef_nscond+1,is) = userdef_ai(userdef_nscond,is)**2 / &
          userdef_ai(userdef_nscond-1,is)
  enddo
  !  
  return
end subroutine userdef_checkall

!-------------------------------------------------------------------------
!                 WRITING ROUTINE FOR USERDEFINED MODULE
!-------------------------------------------------------------------------
subroutine userdef_writeframe(itime)
  implicit none
  integer :: itime
  return
end subroutine userdef_writeframe

!-------------------------------------------------------------------------
!                   OPERATOR-SPLITTED `REACTION RATES'
!-------------------------------------------------------------------------
subroutine userdef_do_reactions()
  implicit none
  !
  double precision :: peq,neq,alp,coef,mh2o,tg
  double precision :: cnd_vapor,dt,tcond_vap,tcond_dust,nvap,teq
  double precision :: tcondevap_dust,dummy,c0,e,tcondevap,time,timenew
  double precision :: condrate_per_area
  integer :: ir,iz,is,im,iss,istep
  doubleprecision :: pi,mp,kk
  parameter(pi  = 3.1415926535897932385d0)
  parameter(mp  = 1.6726d-24)    ! Mass of proton          [g]
  parameter(kk  = 1.3807d-16)    ! Bolzmann's constant     [erg/K]
  !
  ! Set the sticking coefficient, which we take globally constant for now
  !
  alp = 0.9d0
  !
  ! Set the mass of the vapor molecule
  !
  mh2o = (16.d0+2.d0)*mp
  !
  ! Loop over the radial and vertical coordinate
  !
  do ir=1,grid_nr
     do iz=1,grid_nz
        !
        ! Get the gas temperature
        !
        tg = tgas(iz,ir)
        !
        ! Calculate the equilibrium vapor pressure
        !
        peq = exp(-(6070.d0/tg) + 30.86d0)
        !
        ! and the corresponding equilibrium number density
        !
        neq = peq / ( kk*tg )
        !
        ! Compute the coefficient alp / sqrt(2*pi*m*k*T)
        !
        coef = alp / sqrt(2*pi*mh2o*kk*tg)
        !
        ! Compute the source and sink coefficients for the N_ik, i.e.
        ! the left and right `velocities' in mass-space. 
        !
        c0 = coef*mh2o*kk*tg
        e  = coef*mh2o*peq
        if(c0.lt.0.d0) stop 901
        if(e.lt.0.d0) stop 902
        !
        ! Do the sub-time-stepping for the evaporation/condensation
        !
        time = 0.d0
        istep = 1
        do while(time.lt.time_dt)
           !
           ! Check if we are not looping forever
           !
           if(istep.gt.userdef_nstepmax) then
              write(*,*) 'ERROR: Time substepping overflow...'
              stop 701
           endif
           !
           ! Get the vapor number density
           !
           nvap = rhochem(iz,1,ir)
           !
           ! Compute the advection 'velocity' for the grains in 
           ! mu-space (i.e. grain ice mantel mass space).
           !
           condrate_per_area  = c0*nvap - e
           do is=1,userdef_nsrefr
              do im=2,userdef_nscond
                 iss                 = (is-1)*userdef_nscond + im
                 userdef_velo(im,is) = 4*pi*userdef_ai(im,is)**2 * condrate_per_area
              enddo
              userdef_velo(1,is)                = 0.d0
              userdef_velo(userdef_nscond+1,is) = 0.d0
           enddo
           !
           ! Compute the CFL timestep
           !
           tcondevap_dust = 1d99
           do is=1,userdef_nsrefr
              do im=2,userdef_nscond
                 if(userdef_velo(im,is).gt.0.d0) then
                    dummy = ( userdef_mi(im,is) - userdef_mi(im-1,is) ) / &
                         userdef_velo(im,is) 
                 elseif(userdef_velo(im,is).lt.0.d0) then
                    dummy = ( userdef_mi(im,is) - userdef_mi(im+1,is) ) / &
                         userdef_velo(im,is) 
                 else
                    dummy = 1d99
                 endif
                 if(dummy.lt.0.d0) stop 801
                 if(dummy.lt.tcondevap_dust) then
                    tcondevap_dust = dummy
                 endif
              enddo
           enddo
           !
           ! First compute the dust number densities PER unit Delta mu,
           ! because this is what usually goes into an advection 
           ! algorithm, making it easier to implement such algorithms
           ! here.
           !
           do is=1,userdef_nsrefr
              do im=1,userdef_nscond
                 iss                  = (is-1)*userdef_nscond + im
                 userdef_ndust(im,is) = rhodust(iz,iss,ir) /           &
                      ( userdef_mi(im+1,is) - userdef_mi(im,is) )
              enddo
           enddo
           !
           ! Compute 'fluxes' at the cell interfaces. Here we will use
           ! a simple donor-cell scheme for now, though we may/will improve
           ! later.
           !
           do is=1,userdef_nsrefr
              do im=2,userdef_nscond
                 if(userdef_velo(im,is).ge.0.d0) then
                    userdef_flux(im,is) = userdef_velo(im,is) * userdef_ndust(im-1,is)
                 else
                    userdef_flux(im,is) = userdef_velo(im,is) * userdef_ndust(im,is)
                 endif
              enddo
              userdef_flux(1,is)                = 0.d0
              userdef_flux(userdef_nscond+1,is) = 0.d0
           enddo
           !
           ! Compute the 'sources' from the fluxes by differencing
           ! Note: because the 'rhodust' is N_{k,l}, which is already
           !       integrated over the mu mass bin, we do not need
           !       to divide by dm here.
           !
           do is=1,userdef_nsrefr
              do im=1,userdef_nscond
                 userdef_flsrc(im,is) = userdef_flux(im,is) - userdef_flux(im+1,is)
              enddo
           enddo
           !
           ! Compute how much the total netto condensation rate is
           !
           cnd_vapor = 0.d0
           do is=1,userdef_nsrefr
              do im=2,userdef_nscond
                 iss                 = (is-1)*userdef_nscond + im
                 cnd_vapor = cnd_vapor + userdef_flux(im,is) *    &
                      ( dust_m(iss) - dust_m(iss-1)  ) / mh2o
              enddo
           enddo
           !
           ! and the corresponding time scale
           !
           if(cnd_vapor.gt.0.d0) then
              tcond_vap = rhochem(iz,1,ir) / cnd_vapor
           else
              tcond_vap = 1d99
           endif
           !
           ! Compute the time step for the sub-stepping
           !
           tcondevap = min(tcondevap_dust,tcond_vap)
           dt        = userdef_cfl*tcondevap
           if(dt.le.0.d0) then
              write(*,*) 'ERROR: Dt is <=0:   dt=',dt
              write(*,*) tcondevap_dust,tcond_vap
              stop 333
           endif
           !
           ! Check if the time step will overshoot the evap/cond
           ! gas equilibrium. If so, then put the time step precisely
           ! such that it will reach (in the linear approx) exactly
           ! this equilibrium. 
           !
           ! NOTE: This is not really a correct procedure, but it saves
           ! enormous computational time and with a bit of luck the result
           ! is not too bad.
           !
           if((abs((nvap/neq)-1.d0).gt.userdef_dneq).and.        &
                (cnd_vapor.ne.0.d0)) then
              teq = ( nvap - neq ) / cnd_vapor
              if(teq.le.0.d0) teq = 1d99
           else
              teq = 1d99
           endif
           if(teq.lt.dt) dt=teq
           !
           ! Make sure that the time step is right
           !
           timenew = time + dt
           if(timenew.gt.time_dt) then
              dt      = time_dt - time
              timenew = time_dt
           endif
           !
           ! Now do a simplistic integration of the equation for 
           ! the vapor number density
           !
           rhochem(iz,1,ir) = nvap - dt*cnd_vapor
           !
           ! Do the integration of the equation for 
           ! the dust number densities using the 'fluxes'.
           !
           do is=1,userdef_nsrefr
              do im=1,userdef_nscond
                 iss                  = (is-1)*userdef_nscond + im
                 rhodust(iz,iss,ir)   = rhodust(iz,iss,ir) +       &
                      dt * userdef_flsrc(im,is)
              enddo
           enddo
           !
           ! Do basic checks, and replaced numbers that become smaller
           ! than 1d-98 with 0.d0
           !
           if(rhochem(iz,1,ir).lt.0.d0) stop 661
           do iss=1,dust_nrspec
              if(rhodust(iz,iss,ir).lt.0.d0) stop 662
              if(rhodust(iz,iss,ir).lt.1.d-98) rhodust(iz,iss,ir)=0.d0
           enddo
           !
           ! Update the local time
           !
           time  = timenew
           istep = istep + 1
        enddo
        !
     enddo
  enddo
  !
end subroutine userdef_do_reactions

!-------------------------------------------------------------------------
!                          CLEANUP
!-------------------------------------------------------------------------
subroutine userdef_cleanup()
  implicit none
  if(allocated(userdef_velo)) deallocate(userdef_velo)
  if(allocated(userdef_flux)) deallocate(userdef_flux)
  if(allocated(userdef_flsrc)) deallocate(userdef_flsrc)
  if(allocated(userdef_mi)) deallocate(userdef_mi)
  if(allocated(userdef_ai)) deallocate(userdef_ai)
  if(allocated(userdef_ndust)) deallocate(userdef_ndust)
  !
end subroutine userdef_cleanup



end module userdef_module
